@@ -2,6 +2,7 @@
  * Copyright (C) 2013 x265 project
  *
  * Authors: Steve Borho <steve@borho.org>
+ *          Min Chen <chenm003@163.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +31,7 @@
 #include "piclist.h"
 #include "threadpool.h"
 
-namespace x265 {
+namespace X265_NS {
 // private namespace
 
 struct Lowres;
@@ -44,7 +45,6 @@ class Lookahead;
 struct LookaheadTLD
 {
     MotionEstimate  me;
-    ReferencePlanes weightedRef;
     pixel*          wbuffer[4];
     int             widthInCU;
     int             heightInCU;
@@ -103,31 +103,32 @@ public:
     PicList       m_outputQueue;     // pictures to be encoded, in encode order
     Lock          m_inputLock;
     Lock          m_outputLock;
+    Event         m_outputSignal;
+    LookaheadTLD* m_tld;
+    x265_param*   m_param;
+    Lowres*       m_lastNonB;
+    int*          m_scratch;         // temp buffer for cutree propagate
 
     /* pre-lookahead */
     int           m_fullQueueSize;
+    int           m_histogram[X265_BFRAME_MAX + 1];
+    int           m_lastKeyframe;
+    int           m_8x8Width;
+    int           m_8x8Height;
+    int           m_8x8Blocks;
+    int           m_cuCount;
+    int           m_numCoopSlices;
+    int           m_numRowsPerSlice;
+    double        m_cuTreeStrength;
+
     bool          m_isActive;
     bool          m_sliceTypeBusy;
     bool          m_bAdaptiveQuant;
     bool          m_outputSignalRequired;
     bool          m_bBatchMotionSearch;
     bool          m_bBatchFrameCosts;
-    Event         m_outputSignal;
-
-    LookaheadTLD* m_tld;
-    x265_param*   m_param;
-    Lowres*       m_lastNonB;
-    int*          m_scratch;         // temp buffer for cutree propagate
-    
-    int           m_histogram[X265_BFRAME_MAX + 1];
-    int           m_lastKeyframe;
-    int           m_8x8Width;
-    int           m_8x8Height;
-    int           m_8x8Blocks;
-    int           m_numCoopSlices;
-    int           m_numRowsPerSlice;
     bool          m_filled;
-
+    bool          m_isSceneTransition;
     Lookahead(x265_param *param, ThreadPool *pool);
 
 #if DETAILED_CU_STATS
@@ -156,7 +157,7 @@ protected:
     void    slicetypeAnalyse(Lowres **frames, bool bKeyframe);
 
     /* called by slicetypeAnalyse() to make slice decisions */
-    bool    scenecut(Lowres **frames, int p0, int p1, bool bRealScenecut, int numFrames, int maxSearch);
+    bool    scenecut(Lowres **frames, int p0, int p1, bool bRealScenecut, int numFrames);
     bool    scenecutInternal(Lowres **frames, int p0, int p1, bool bRealScenecut);
     void    slicetypePath(Lowres **frames, int length, char(*best_paths)[X265_LOOKAHEAD_MAX + 1]);
     int64_t slicetypePathCost(Lowres **frames, char *path, int64_t threshold);
