@@ -36,7 +36,7 @@
 #endif
 #endif
 
-using namespace x265;
+using namespace X265_NS;
 using namespace std;
 
 YUVInput::YUVInput(InputFileInfo& info)
