@@ -53,7 +53,7 @@ bool YUVOutput::writePicture(const x265_picture& pic)
     uint64_t fileOffset = pic.poc;
     fileOffset *= frameSize;
 
-    X265_CHECK(pic.colorSpace == colorSpace, "invalid color space\n");
+    X265_CHECK(pic.colorSpace == colorSpace, "invalid chroma subsampling\n");
     X265_CHECK(pic.bitDepth == (int)depth, "invalid bit depth\n");
 
 #if HIGH_BIT_DEPTH
